/*
	Simple TextBox
	by Pat Niemeyer (pat@icon-stl.net)
	
	Supports configurable font, rows, cols, scroll-back,
	colored text, automagic links, and line wrapping.

*/

package text;

import java.util.*;
import awt.*;
import browser.Applet;
import text.*;
import net.www.html.URL;

public class TextBox extends TextDisplay {
	TextLine[] text;
	int rows, cols, xpad, ypad, backlines = 0, lines;
	boolean SCROLLING=false;	// for future use
	int scrolloff, lastscrolloff;
	public TextNotice notice;

    public TextBox( Applet mom, Font f, int r, int c, int back, int x, int y ) {
		super( mom, f, x, y );

		rows = r; cols = c; 
		backlines=back;
		lines = rows+back;
		Width = font.stringWidth(maxwidestring(cols) + "@@");
		xpad = font.stringWidth("@")/2;
		Height = font.actualHeight * (rows+1);
		ypad = font.actualHeight/2;
		text = new TextLine[lines];

		for (int i = 0; i < lines; i++)
			text[i] = new TextLine();

    }

	public void wait_notice( String str ) {
		notice = new TextNotice( this, str );
		notice.start_spinner();
		repaint();
	}

	public void notice( String str ) {
		notice = new TextNotice( this, str );
		repaint();
	}

	public void notice( ) {
		notice.stop_spinner();
		notice = null;
		repaint();
	}

	// I have children so I have to catch update
	public void update(Graphics g) {
		super.update(g);
		if ( notice != null )
			notice.update(g);
	}

	public void paint(Graphics g) {
		TextLine line;

		drawbox(g);
		clear(g);

		if ( notice != null ) {
			notice.paint(g);
			return;
		}

		g.setFont(font);

		int y = Height - font.descent - ypad;
		for (int i = 0; i < rows; i++) {
			line = text[i+scrolloff];

			if (line.color != null)
				g.setForeground(line.color);
			else
				g.setForeground(Color.black);

			g.drawString(line.str, xo + xpad, yo + y);

			for (Enumeration e=line.hlinks.elements(); e.hasMoreElements(); ) {
				TextHLink hl = (TextHLink)e.nextElement();
				g.setForeground(hl.color);
				g.drawString(hl.str, xo+xpad+hl.xpos, yo+y);
			}
				
			y -= font.actualHeight;
		}
	}

	public void put(String str) {
		put(null, str);
	}

    synchronized public void put(Color color, String str) {
		String next = null;

		if ( str.length() > cols ) {
			next = str.substring(cols-1);
			str=str.substring(0, cols-1);
		}

		for (int i=(lines-1); i > 0; i--)
			text[i] = text[i-1];

		text[0] = new TextLine( str, color, parsehlinks(str) );

		if ( next != null )
			put(color, next);
		
		repaint();
    }

	private Vector parsehlinks(String str) {
		int start = 0, end, x, w;
		String s;
		Vector v = new Vector();

		while ( (start=str.toLowerCase().indexOf("http:", start)) != -1 ) {
			end = str.indexOf(" ", start);
			if (end == -1)
				end=str.length();
			s = str.substring(start, end);
			w = font.stringWidth( s );
			x = font.stringWidth( str.substring(0, start) );
			v.addElement( new TextHLink(s, x, w) );
			start = end;
		}

		return v;
	}

	private int downx, downy;
	public void mouseDown( int x, int y ) {
		if ( !mouseIn(x,y) )
			return;

		downx = x;
		downy = y;
		SCROLLING=true;
	}

	public void mouseUp( int x, int y ) {
		if ( !mouseIn(x,y) )
			return;

		if ( (x == downx) && (y == downy) ) {
			mouseClick( x, y );
			return;
		}

		scrolloff=0;
		SCROLLING=false;
		repaint();
	}

	public void mouseClick( int x, int y ) {
		TextHLink hl;
		int row = rows - 1 - (y - ypad)/font.actualHeight + scrolloff;
		if ( (hl=text[row].linkAtX( x )) != null ) {
			System.out.println("Clicked on link: "+hl.str);
			hl.color = new Color(0,0,190); // dark blue
			repaint();
			URL url = new URL(hl.str);
			((browser.WRWindow)applet.item.parent).pushURL(url);
		}
		
	}

	public void mouseDrag( int x, int y ) {
		if ( !mouseIn(x,y) || ( notice != null ) )
			return;

		scrolloff = backlines*(y-downy)/Height ;

		if (scrolloff < 0)
			scrolloff = 0;

		if (scrolloff > backlines)
			scrolloff = backlines;

		if (lastscrolloff != scrolloff) {
			lastscrolloff = scrolloff;
			repaint();
		}
	}

}
